PRCS 1.2 for BeOS R4.
Ported September-November 1998 by Lars Duening <lars@cableinet.co.uk>.

  PRCS, the Project Revision Control System, is a set of tools that (like
  CVS) provide a way to deal with sets of files and directories as an
  entity, preserving coherent versions of the entire set. Its purpose is
  similar to that of SCCS, RCS, and CVS, but (according to its authors, at
  least), it is much simpler than any of those systems.

  Please read the original documentation - this file only deals with the
  specialities of the Be port.

  If you find bugs, or have comments or improvements, I'm glad to hear
  from you.


Requirements
------------

  A machine (PowerPC or x86) running BeOS R3 or better to run PRCS.
  PRCS needs RCS 5.7 and the GNU diffutils. Though both come preinstalled
  with BeOS, the standard RCS contains a serious bug which must be fixed
  in order for PRCS to work.

  The full Metrowerks Codewarrior to compile the source distribution on R3.

  The GeekGadget makeinfo and texi2html to create the documentation.

  The GeekGadget autoconf/-make/-header suite to configure the source
  distribution.

  When used, PRCS needs about 3.5 KByte memory for every file checked
  in, and (temporarily) the same amount again to fork subprocesses.


Distributions
-------------

  PRCS comes in one of three packages:

  The original source distribution from the PRCS main site (1.2.10 or newer).
  
    This distribution contains just the plain sources. To create a
    binary, it has to be configured and compiled.

  The Beified source distribution from BeWare.

    This distribution contains just the plain sources, but already
    configured for Be (no autoconf necessary).

  The binary distribution from BeWare.

    This distribution contains the precompiled binaries and the
    documentation in all sorts of formats.


Installation
------------

  If you are using one of the source distributions, you have to compile
  the binary first (this also makes the documentation). For instructions
  on how to do this, see the next paragraph.
  
  The binary distribution comes with precompiled binaries for PowerPC
  and Intel x86.

  The binaries are located in the directories src/obj.ppc and src/obj.x86.
  To install them on your system, simply copy the binary of your choice
  into a directory in your shell search path (/boot/home/config/bin is
  probably most appropriate). With the command

    prcs config

  you can then test if prcs is happy where it is.

  The documentation is in the doc/ subdirectory and comes at least
  as man-page, as html files and as texinfo file. Put them whereever
  you keep your online documentation.

  IMPORTANT: The RCS package as delivered with R4 contains a nasty
  bug which can cause its programs (notably ci) to crash. To correct
  that bug, unpack /boot/optional/gnu/gnu.tgz, apply the patch
  be/rcslex.diff in the subdirectory rcs-5.7/src and recompile the
  rcs package. That done, copy the created executables to /bin.
  If the patch already was applied, this all is of course not
  necessary.


Compilation
-----------

  The source distributions come in two flavours: the original source
  distribution from the PRCS author, and the already configured
  distribution from BeWare. Where the latter can be made directly, the
  former _has_ to be configured first.

  To (re)configure PRCS, you need the GeekGadget tools autoconf, automake
  and autoheader. The actual configuration is taken care of by the
  script 'config-be' in the top directory. It is not advisable to
  call 'configure' directly as the created Makefiles need some after-
  patching to become useful. To configure the original source distribution,
  the argument '-f' must be given on the first call to config-be - it
  causes the necessary recreation of the configure script. For further
  information about config-be, give the argument '--help'.

  To rebuild the PRCS package, you need the full version of the Metrowerks
  Codewarrior, and the GeekGadget tools makeinfo and texi2html for the
  documentation. The whole package is rebuilt by giving the command 'make'
  in the top directory. To just recompile the binary, cd into the src/
  directory and command 'make' there.

  Depending on the type of Makefile chosen via config-be, the binary
  is created in one of two places. The default Makefile creates the
  binaries (and object files) directly in src/, whereas the special
  Be-Makefile (created with 'config-be --be') puts these files into
  the directory src/obj.<cpu>/, with <cpu> being either 'x86' or 'ppc'.

  In future, the Be-Makefile will also offer the possibility of cross-
  compilation by giving the argument 'CPU=<cpu>' to the make command.

  
About the port
--------------

  Porting prcs was quite easy. Besides modifying the sources to use the
  standard C++ library instead of the Gnu C++ library, most of the work
  went into the (auto)configuration and bug hunting.

  In detail, the following problems had to be solved:

   - the RCS distributed with BeOS contains a serious bug in the
     memory handling.

   - PRCS optimizes the calls to RCS by passing as many files per call
     as possible. However, the internal handling of child processes
     lead to a deadlock as PRCS assumes that it can wait until a child
     terminates before it reads their output, the limited buffer for
     IPC on the other hand means that the child processes need PRCS to
     read their output even before they terminate. The solution was to
     force PRCS to check on its children every second (this might be
     addressed in later PRCS versions on a general level).

   - PRCS uses an alarm signal handler to keep its .lock files up to date.
     Unfortunately the handler uses non-thread safe functions which on
     Be can cause the program to fail or, worse, to freeze. Fortunately
     it is possible to cautiously use threads in conjunction with fork(),
     so the alarm signal was replaced by a parallel thread snoozing
     for the allocated time and then calling the alarm signal handler
     in a well-defined thread-safe manner.

     Thanks to Chris Herborth (<chrish@qnx.com>) and Jon Watte
     (<hplus@zilker.net>) for their helpful comments on this subject
     
   - index() and friends are defined in <posix/bsd_mem.h> as macros
     to strchr() etc; bsd_mem.h is included as part of the iostream
     definitions. This breaks classes which define 'index' functions :-(
     Fix: #define USG on the compiler commandline.
     
   - Be include <cpp/mcompile.h> defines READ_LOCK() and others, clashing
     with the READ_LOCK define in lock.h.
     Fix: change lock.h to undef READ_LOCK and WRITE_LOCK.
     
   - BeOS has no itimer, it was simulated with the alarm thread
     described above.
   
   - BeOS up to R4 inclusive can't select on files. syscmd.cc is affected
     by this, I poll instead :-( This also has the effect that when
     waiting for the last output of a terminated child, the process returns
     too early from the reading function and thus loses data. The
     solution for this was to introduce a loop which only terminates
     if all pipes to the ex-child are definitely closed.
     
   - The Makefiles created by the GeekGadget auto{conf,make} packages
     are incomplete and geared towards the gnu compiler. As described
     above, I put a wrapper around the configure mechanism and do
     some patching on the Makefiles.
     
   - The configure created by autoconf fails to determine a few
     properties of the BeOS environment:
       - the type 'ino_t' is expected by inclusion of <stdio.h>, under
         BeOS <sys/types.h> has to be included explicitely.
       - the type 'fd_set' is hidden in <socket.h>, not where
         configure.in expects it to be.
       - signal handler must be compiled with 'int' as argument type,
         else the generated code will mess up the stack.
     The solution in all cases was to give configure.in explicite
     knowledge about the BeOS platform.


TODO
----

  PRCS knows nothing about Be's file attributes. As it relies on rcs
  for the file handling, this is tricky to solve. It might be better
  to wait until PRCS 2 with this, but I'm working on it.

  Cross-compilation support on PowerPC platforms.
